/* 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * *****
 *
 * Notes:
 *
 *    Since we can't know ahead of time where Opera will be installed, we Kiboze the
 *    Start Menu tree.  If the shortcut has been renamed (i.e. it is no longer
 *    titled "Opera"), we're out of luck.  The only option at that
 *    point would be to open and dereference every symlink we find, and see if the
 *    target leaf name is "opera.exe".  Not doing that at the moment, since
 *    it just seems insane.
 */

var operaviewMenuItems = new Array("operaview-do-view", "operaview-do-viewlink");
var userPrograms = "Progs";
var allUserPrograms = "CmPrgs";
var applicationData = "AppData";

var gOperaViewBundle;

var hexVals = new Array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
              "A", "B", "C", "D", "E", "F");

// deleted these chars from the include list ";", "/", "?", ":", "@", "=", "&" and #
// so that we could analyze actual URLs
var unsafeString = "\"<>%\\^[]`\+\$\,";

function changeIt(val)
{
	var len     = val.length;
	var i       = 0;
	var newStr  = "";

	for (i=0; i < len; i++)
	{
		if (val.substring(i, i+1).charCodeAt(0) < 255)  // hack to eliminate the rest of unicode from this
		{
			if (isUnsafe(val.substring(i, i+1)) == false)
				newStr = newStr + val.substring(i, i+1);
			else
				newStr = newStr + convert(val.substring(i, i+1));
		}
		else // unicode character
			newStr = newStr + val.substring(i, i+1);
	}
	
	return newStr;
}

// this converts a given char to url hex form
function convert(val)
{
	return  "%" + decToHex(val.charCodeAt(0), 16);
}

// part of the hex-ifying functionality
function decToHex(num, radix)
{
	var hexString = "";
	while (num >= radix)
	{
		temp = num % radix;
		num = Math.floor(num / radix);
		hexString += hexVals[temp];
	}
	hexString += hexVals[num];
	return reversal(hexString);
}

// part of the hex-ifying functionality
function reversal(s)
{
	var len = s.length;
	var trans = "";
	for (i=0; i<len; i++)
		trans = trans + s.substring(len-i-1, len-i);

	s = trans;
	return s;
}

// this function checks to see if a char is URL unsafe.
// Returns bool result. True = unsafe, False = safe
function isUnsafe(compareChar)
{
	if (unsafeString.indexOf(compareChar) == -1 && compareChar.charCodeAt(0) > 32 && compareChar.charCodeAt(0) < 123)
		return false; // found no unsafe chars, return false
	else
		return true;
}

function operaviewContext()
{
	if(gContextMenu)
	{
		var menuitem = document.getElementById(operaviewMenuItems[0]);
		var menuitemLink = document.getElementById(operaviewMenuItems[1]);

		if (gContextMenu.onLink)
		{
			var href;
			if (gContextMenu && gContextMenu.getLinkURL)
				href = gContextMenu.getLinkURL();
			else if (gContextMenu && gContextMenu.linkURL)
				href = gContextMenu.linkURL();
								
			if (href.indexOf('http://') != 0 && href.indexOf('https://') != 0 && href.indexOf('ftp://') != 0 && href.indexOf('file://') != 0)
				menuitemLink.hidden = true;
			else
				menuitemLink.hidden = false;

			menuitem.hidden = true;
		}
		else
		{
			if (gContextMenu.isTextSelected || gContextMenu.onImage || gContextMenu.onTextInput)
				menuitem.hidden = true;
			else if (gBrowser.currentURI.spec.indexOf('http://') != 0 && gBrowser.currentURI.spec.indexOf('https://') != 0 && gBrowser.currentURI.spec.indexOf('ftp://') != 0 && gBrowser.currentURI.spec.indexOf('file://') != 0)
				menuitem.hidden = true;
			else
				menuitem.hidden = false;
			menuitemLink.hidden = true;
		}
	}
}

function operaView() {
  if(gContextMenu) {
     var href = gBrowser.currentURI.spec;

	if (href.indexOf('file://') != 0)
  	href = changeIt(href);
    
 	operaViewLaunch("Opera.lnk", href);
  }
}


function operaViewLink() {
  if(gContextMenu) {
  
    var href;
    
	  if (gContextMenu && gContextMenu.getLinkURL)
  		href = gContextMenu.getLinkURL();
	  else if (gContextMenu && gContextMenu.linkURL)
  		href = gContextMenu.linkURL();

		if (href.indexOf('file://') != 0)
  		href = changeIt(href);
      
    operaViewLaunch("Opera.lnk", href);
  }
}



// attempt to grab the real path of a predefined directory
//
function tryDir(dsp, key)
{
   try
   {
      var nif = dsp.get(key, Components.interfaces.nsIFile);
      return(nif.path);
   }
   catch (ar)
   {
      return("");
   }
}


function searchPath(path, fname)
{
   var result = null;

   try
   {
      var f = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);

      f.initWithPath(path);

      if (f.exists() && f.isDirectory())
      {
         var entries = f.directoryEntries;

         while (entries.hasMoreElements())
         {
            var ent = entries.getNext().QueryInterface(Components.interfaces.nsIFile);

            if (ent.isDirectory())
            {
               result = searchPath(ent.path, fname);

               if (result)
               {
                  break;
               }
            }
            else if (ent.isSymlink())
            {
               if (ent.leafName.toLowerCase() == fname.toLowerCase())
               {
                  result = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
                  result.followLinks = true;
                  result.initWithPath(ent.path);

                  if (result.target == "")
                  {
                     result = null;
                  }
                  else
                  {
                     break;
                  }
               }
            }
            else if (ent.isFile())
            {
               if (ent.leafName.toLowerCase() == fname.toLowerCase())
               {
                  result = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
                  result.initWithPath(ent.path);
                  break;
               }
            }
         }
      }
   }
   catch (ar)
   {
      return(null);
   }

   return( result );
}



function operaViewLaunch (path,argumentstext)
{
    var cantMessage = "can't find";
    var badPath = "bad path";

    try
    {
        cantMessage = gOperaViewBundle.getString("operaview.cantFindExplorer");
        badPath = gOperaViewBundle.getString("operaview.badPath");
    }
    catch(e)
    {
        alert(e);
    }

  var prefservice = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefService);   

  var prefs = prefservice.getBranch("");

   try{
      if(path=="") return false;

      var operaloc = null;

      if (prefs.getPrefType("operaview.operaapp") == prefs.PREF_STRING)
      {
				try {
	        operaloc = prefs.getCharPref("operaview.operaapp");
				} catch(e) {
						operaloc = null;
					}
      }

      var   natTarget = null;
      var   usePath = null;

      if (operaloc != null)
      {
         natTarget = operaloc;
      }
      else
      {
         var dsprops = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);

         usePath = tryDir(dsprops, userPrograms);    // try user-specific program menu first

         var file = null;

         if (usePath != "")
         {
            file = searchPath(usePath, path);
         }

         if (! file)
         {
            usePath = tryDir(dsprops, allUserPrograms);   // no joy?  try "all users" program menu

            if (usePath != "")
            {
               file = searchPath(usePath, path);
            }
         }

         // (almost) last ditch -- find the windows directory
         // assume that the main Program Files directory is on the same drive
         // look in there, under Program Files\Opera, for opera.exe
         //
         if (! file)
         {
            var usePath = tryDir(dsprops, "WinD");

            if ((usePath != "") && (usePath.charAt(1) == ":"))
            {
               usePath = usePath.substr(0, 2) + "\\program files\\opera";

               file = searchPath(usePath, "opera.exe");
            }
         }

				 // the very last ditch -- try to run just 'opera'         
         if (! file)
         {
            file = searchPath(".", "opera");
         }

         if ((! file) || (! file.exists()))
         {
            alert(cantMessage);
            return false;
         }

         natTarget = file.target;
      }

      var targetFile = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);

			try {
	      targetFile.initWithPath(natTarget);

	      if (! targetFile.exists())
  	    {
    	     alert(cantMessage);
      	   return(false);
	      }

	      //usePath = targetFile.target;

	      var process = Components.classes['@mozilla.org/process/util;1'].getService(Components.interfaces.nsIProcess);
  	    process.init(targetFile);
    	  var arguments= [] ;

	      arguments.push(argumentstext);

	      process.run(false, arguments, arguments.length, {});
	      
	      return true;
			} catch(e) {
					alert(badPath); return false;
			  }
   }catch(e){ 
      alert(e);return false;
   }

   return false;		// avoid JavaScript Error.
}


function operaviewInit() {
  var menu = document.getElementById("contentAreaContextMenu");
  menu.addEventListener("popupshowing",operaviewContext,false);

  gOperaViewBundle = document.getElementById("bundle_operaview");

  if (! gOperaViewBundle)
  {
      alert("no bundle");
  }
  
  var tbutton = document.getElementById("operaview-button-1");
	if (tbutton)
	{
		tbutton.setAttribute('disabled', 'true');
		tbutton.disabled = true;
	}
  
  var appcontent = document.getElementById("appcontent");   // browser
  if (appcontent)
  {
  	appcontent.addEventListener("load", operaviewLoad, true);
  }
  
  try {
			const nsISupportsString = Components.interfaces.nsISupportsString;
    
			var pref = Components.classes["@mozilla.org/preferences-service;1"]
									.getService(Components.interfaces.nsIPrefService)
									.getBranch("extensions.{87f54a61-c9b3-4138-a38a-33c31770bb9e}.");
    
			var str = Components.classes["@mozilla.org/supports-string;1"]
									.createInstance(nsISupportsString);
    
			str.data = gOperaViewBundle.getString("operaview.description");
            
			pref.setComplexValue("description", nsISupportsString, str);
	} catch (err) {}

}

function operaviewLoad(aEvent)
{
	var doc = aEvent.originalTarget;
	var href = '';
	
	if (doc.location && doc.location.href)
	{
		href = doc.location.href;
	}
	
	var tbutton = document.getElementById("operaview-button-1");
	if (tbutton)
	{
		if (href.indexOf('http://') != 0 && href.indexOf('https://') != 0 && href.indexOf('ftp://') != 0 && href.indexOf('file://') != 0)
			enabled = false;
		else
			enabled = true;
		tbutton.setAttribute('disabled', enabled ? 'false' : 'true');
		tbutton.disabled = !enabled;
	}
}

function setOperaviewOptions()
{
		var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                               .getService(Components.interfaces.nsIPrefService);
    
    if (document.getElementById('operaloc').value)
    {
				var prefs = prefService.getBranch("");
				prefs.setCharPref("operaview.operaapp", document.getElementById('operaloc').value);
    }
    
    if (document.getElementById('showicons') != null)
    {
			var prefs = prefService.getBranch("");
			prefs.setBoolPref("operaview.showicons", document.getElementById('showicons').checked);
    }
    
    window.close();
}

function pickOpera()
{
    var picker = Components.classes["@mozilla.org/filepicker;1"].getService(Components.interfaces.nsIFilePicker);

    picker.init(window, "OperaView", 0);
    picker.appendFilters(64);
    
    if (picker.show() == 0)
    {
        document.getElementById('operaloc').value = picker.file.target;
    }
}


function init()
{
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);   
    
    var prefs = prefservice.getBranch("");

    var operaloc = null;

    if (prefs.getPrefType("operaview.operaapp") == prefs.PREF_STRING)
    {
      operaloc = prefs.getCharPref("operaview.operaapp");
    }

    if ((operaloc != null) && (operaloc.length > 0)) 
    {
        document.getElementById('operaloc').value = operaloc;
    }
    
    var opera_showicons = null;
    
    if (prefs.getPrefType("operaview.showicons") == prefs.PREF_BOOL)
    {
      opera_showicons = prefs.getBoolPref("operaview.showicons");
    }

    if (opera_showicons != null) 
    {
        document.getElementById('showicons').checked = opera_showicons;
    }
    else
    {
        document.getElementById('showicons').checked = false;
    }

    prefs = null;
}

// do the init on load
window.addEventListener("load", operaviewInit, false); 